#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;
#pragma link off all namespaces;


#pragma link C++ namespace mattak+; 
#pragma link C++ namespace mattak::k+; 
#pragma link C++ namespace mattak::convert+; 

#pragma link C++ class mattak::Header+;
#pragma link C++ class mattak::IWaveforms+;
#pragma link C++ class mattak::Waveforms+;
#pragma link C++ class mattak::CalibratedWaveforms+;
#pragma link C++ struct mattak::WaveformPlotOptions+;
#pragma link C++ class mattak::VoltageCalibration+;
#pragma link C++ class mattak::Pedestals+;
#pragma link C++ class mattak::TriggerInfo+;
#pragma link C++ class mattak::RadiantTriggerInfo+;
#pragma link C++ class mattak::LTTriggerInfo+;
#pragma link C++ class mattak::Sensors+;
#pragma link C++ class mattak::LTEStats+;
#pragma link C++ class mattak::GNSS+;

#pragma link C++ class mattak::RunInfo+;
#pragma link C++ class mattak::FirmwareVersion+;
#pragma link C++ class mattak::FlowerGainCode+;

#pragma link C++ class mattak::GNSS::Sat+;
#pragma link C++ class mattak::DAQStatus+;
#pragma link C++ class mattak::LTScalerGroup+;
#pragma link C++ class mattak::CalpulserInfo+;
#pragma link C++ class mattak::RadiantVoltages+;
#pragma link C++ enum mattak::CalpulserOutput; 
#pragma link C++ enum mattak::CalpulserMode; 


#pragma link C++ class mattak::Dataset+;
#pragma link C++ struct mattak::DatasetOptions+;
